<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Connections
    |--------------------------------------------------------------------------
    |
    | This array stores the connections that are added to Adldap. You can add
    | as many connections as you like.
    |
    | The key is the name of the connection you wish to use and the value is
    | an array of configuration settings.
    |
    */
    'connections' => [

        'default' => [

            /*
            |--------------------------------------------------------------------------
            | Auto Connect
            |--------------------------------------------------------------------------
            |
            | If auto connect is true, anytime Adldap is instantiated it will automatically
            | connect to your AD server. If this is set to false, you must connect manually
            | using: Adldap::connect().
            |
            */

            'auto_connect' => true,

            /*
            |--------------------------------------------------------------------------
            | Connection
            |--------------------------------------------------------------------------
            |
            | The connection class to use to run operations on.
            |
            | You can also set this option to `null` to use the default connection class.
            |
            | Custom connection classes must implement \Adldap\Contracts\Connections\ConnectionInterface
            |
            */
            'connection' => Adldap\Connections\Ldap::class,

            /*
            |--------------------------------------------------------------------------
            | Schema
            |--------------------------------------------------------------------------
            |
            | The schema class to use for retrieving attributes and generating models.
            |
            | You can also set this option to `null` to use the default schema class.
            |
            | Custom schema classes must implement \Adldap\Contracts\Schemas\SchemaInterface
            |
            */
            'schema' => Adldap\Schemas\ActiveDirectory::class,

            /*
            |--------------------------------------------------------------------------
            | Connection Settings
            |--------------------------------------------------------------------------
            |
            | This connection settings array is directly passed into the Adldap constructor.
            |
            | Feel free to add or remove settings you don't need.
            |
            */

            'connection_settings' => [

                /*
                   |--------------------------------------------------------------------------
                   | Account Prefix
                   |--------------------------------------------------------------------------
                   |
                   | The account prefix option is the prefix of your user accounts in AD.
                   |
                   | For example, if you'd prefer your users to use only their username instead
                   | of specifying a domain ('ACME\jdoe'), enter your domain name.
                   |
                   */
                'account_prefix' => '',

                /*
                |--------------------------------------------------------------------------
                | Account Suffix
                |--------------------------------------------------------------------------
                |
                | The account suffix option is the suffix of your user accounts in AD.
                |
                | For example, if your domain DN is DC=corp,DC=acme,DC=org, then your
                | account suffix would be @corp.acme.org. This is then appended to
                | then end of your user accounts on authentication.
                |
                */

                'account_suffix' => '@drpgroup',

                /*
                |--------------------------------------------------------------------------
                | Domain Controllers
                |--------------------------------------------------------------------------
                |
                | The domain controllers option is an array of servers located on your
                | network that serve Active Directory. You can insert as many servers or
                | as little as you'd like depending on your forest (with the
                | minimum of one of course).
                |
                */

                'domain_controllers' => ['dc1.drpgroup.private'],

                /*
                |--------------------------------------------------------------------------
                | Port
                |--------------------------------------------------------------------------
                |
                | The port option is used for authenticating and binding to your AD server.
                |
                */

                'port' => 389,

                /*
                |--------------------------------------------------------------------------
                | Timeout
                |--------------------------------------------------------------------------
                |
                | The timeout option allows you to configure the amount of time in
                | seconds that your application waits until a response
                | is received from your LDAP server.
                |
                */
                'timeout' => 5,

                /*
                |--------------------------------------------------------------------------
                | Base Distinguished Name
                |--------------------------------------------------------------------------
                |
                | The base distinguished name is the base distinguished name you'd like
                | to perform operations on. An example base DN would be DC=corp,DC=acme,DC=org.
                |
                | If one is not defined, then Adldap will try to find it automatically
                | by querying your server. It's recommended to include it to
                | limit queries executed per request.
                |
                */

                'base_dn' => 'ou=drpgroup, dc=drpgroup, dc=private',

                /*
                |--------------------------------------------------------------------------
                | Administrator Account Suffix
                |--------------------------------------------------------------------------
                |
                | This option allows you to set a different account suffix for your
                | configured administrator account upon binding.
                |
                | If left empty, your `account_suffix` option will be used.
                |
                */
                'admin_account_suffix' => '@drpgroup',

                /*
                |--------------------------------------------------------------------------
                | Administrator Username & Password
                |--------------------------------------------------------------------------
                |
                | When connecting to your AD server, an administrator username and
                | password is required to be able to query and run operations on
                | your server(s). You can use any user account that has
                | these permissions.
                |
                */

                'admin_username' => 'Directory',
                'admin_password' => env('LDAP_PASSWORD', 'ask a sysadmin'),

                /*
                |--------------------------------------------------------------------------
                | Follow Referrals
                |--------------------------------------------------------------------------
                |
                | The follow referrals option is a boolean to tell active directory
                | to follow a referral to another server on your network if the
                | server queried knows the information your asking for exists,
                | but does not yet contain a copy of it locally.
                |
                | This option is defaulted to false.
                |
                */

                'follow_referrals' => false,

                /*
                |--------------------------------------------------------------------------
                | SSL & TLS
                |--------------------------------------------------------------------------
                |
                | If you need to be able to change user passwords on your server, then an
                | SSL or TLS connection is required. All other operations are allowed
                | on unsecured protocols. One of these options are definitely recommended
                | if you have the ability to connect to your server securely.
                |
                */

                'use_ssl' => false,
                'use_tls' => false,

                /*
                |--------------------------------------------------------------------------
                | SSO (Single Sign On)
                |--------------------------------------------------------------------------
                |
                | If you enable single sign on, be sure you've properly set it up
                | on your server before hand.
                |
                */

                'use_sso' => false,

            ],

        ]
    ],

    /*
           |--------------------------------------------------------------------------
           | Allowed groups
           |--------------------------------------------------------------------------
           |
           | These are arrays that control which groups are allowed to login.
           | A user needs to be in at least 1 of these groups to be able to login, by
           | default. You can change this with the inclusive_groups config option.
           |
           | Groups rank priority is based on their array index when doing the
           | needs_previous checks. So the first element is highest rank.
           |
           */

    'allowed_groups' => [
        'developer' => ['Digital Media'],
        'super' => ['all'],
        'standard' => ['all'],
    ],

    /*
    |--------------------------------------------------------------------------
    | Inclusive groups
    |--------------------------------------------------------------------------
    |
    | This enforces that the user must be in all the ldap groups specified in
    | one of the groups in allowed_groups.
    |
    */

    'inclusive_groups' => true,

    /*
    |--------------------------------------------------------------------------
    | Needs previous groups
    |--------------------------------------------------------------------------
    |
    | When a user logs in to find out which group they are a member of. It will
    | make sure you are able to be the lower ranked groups first.
    |
    | E.g. To become a developer you need to have all the ldap groups from
    | the super role.
    |
    */

    'needs_previous' => false,
];