<?php

namespace Drp\LdapAuthentication;

use Adldap\Laravel\AdldapAuthUserProvider;
use Adldap\Models\User as AdLdapUser;
use Drp\LdapAuthentication\Events\LdapUserGroupEvent;
use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Contracts\Hashing\Hasher;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Arr;

/**
 * Class DrpldapAuthUserProvider
 * @package Drp\LdapAuthentication
 */
class DrpldapAuthUserProvider extends AdldapAuthUserProvider
{
    /**
     * Class for finder the best group for a user.
     * @var GroupFinder
     */
    protected $groupFinder;

    /**
     * Chosen group of the ldap user.
     * @var string
     */
    protected $chosenGroup;

    /**
     * Create a new database user provider.
     * @param Hasher $hasher Hashing alogirthm to use.
     * @param string $model Model of the user to auth.
     * @param GroupFinder $groupFinder A group finder instance.
     */
    public function __construct(Hasher $hasher, $model, GroupFinder $groupFinder)
    {
        $this->model = $model;
        $this->hasher = $hasher;
        $this->groupFinder = $groupFinder;
    }

    /**
     * {@inheritdoc}
     */
    public function validateCredentials(Authenticatable $user, array $credentials)
    {
        // Check if we already have an authenticated AD user.
        if ($this->user instanceof AdLdapUser) {
            return true;
        }

        if ($this->getLoginFallback() && $user->exists) {
            // If the user exists in our local database already and fallback is
            // enabled, we'll perform standard eloquent authentication.
            return parent::validateCredentials($user, $credentials);
        }

    }

    /**
     * Creates a local User from Active Directory.
     * @param AdLdapUser $authenticatedUser The Adldap user to convert.
     * @param string $password The password used to login.
     * @return \Illuminate\Database\Eloquent\Model|null
     */
    protected function getModelFromAdldap(AdLdapUser $authenticatedUser, $password = null)
    {
        // Check if the user is in the right group before building the model
        if (!$this->checkLDAPGroups($authenticatedUser)) {
            return null;
        }

        $eloquentUser = $this->getModelFromAdldapWithTrashed($authenticatedUser, $password);

        $this->saveModel($eloquentUser);

        //If the user is in a group they may need restoring to group access
        if (!is_null($this->chosenGroup)) {
            $this->restoreUserIfGroupAccess($eloquentUser);
        }

        event(new LdapUserGroupEvent($this->chosenGroup, $eloquentUser));

        return $eloquentUser;
    }

    /**
     * Creates a local User from Active Directory.
     * Include any deleted users as they may need re-instating
     * @param AdLdapUser $authenticatedUser The Adldap user to convert.
     * @param string $password The password used to login.
     * @return \Illuminate\Database\Eloquent\Model|null
     */
    protected function getModelFromAdldapWithTrashed(AdLdapUser $authenticatedUser, $password = null)
    {
        list($key, $username) = $this->getEloquentLookupCredentials($authenticatedUser);

        $eloquentUser = $this->retrieveOrCreateEloquentUser($key, $username);

        // Set the username in case of changes in active directory.
        $eloquentUser->{$key} = $username;

        $eloquentUser = $this->syncModelAndPassword($authenticatedUser, $eloquentUser, $password);

        // Bind the Adldap model to the eloquent model (if enabled).
        $eloquentUser = $this->getBindUserToModel() ?
            $this->bindAdldapToModel($authenticatedUser, $eloquentUser) :
            $eloquentUser;

        $eloquentUser->setLdapUser(true);

        return $eloquentUser;

    }

    /**
     * Syncs the models password and attributes
     * @param AdLdapUser $authenticatedUser
     * @param $eloquentUser
     * @param null $password
     * @return Model
     */
    protected function syncModelAndPassword(AdLdapUser $authenticatedUser, $eloquentUser, $password = null)
    {
        // Sync the users password (if enabled). If no password is
        // given, we'll assign a random 16 character string.
        $eloquentUser = $this->syncModelPassword($eloquentUser, $password ?: str_random());

        // Synchronize other active directory attributes on the model.
        return $this->syncModelFromAdldap($authenticatedUser, $eloquentUser);
    }

    /**
     * Return the found Eloquent user or an empty User model
     * @param $key The lookup key
     * @param $username The username attribute
     * @return Model The User model
     */
    protected function retrieveOrCreateEloquentUser($key, $username)
    {
        // Try to retrieve the model from the model key and AD username.
        //Include any thrashed models as they may now have group access enabled
        $eloquentUser = $this->createModel()->newQuery()->withTrashed()->where([$key => $username])->first();

        // Create the model instance of it isn't found.
        return $eloquentUser instanceof Model ? $eloquentUser : $this->createModel();
    }

    /**
     * Return the key and username attributes to be used in the eloquent user lookup
     * @param AdLdapUser $authenticatedUser
     * @return array
     */
    protected function getEloquentLookupCredentials(AdLdapUser $authenticatedUser)
    {
        $attributes = $this->getUsernameAttribute();
        $key = key($attributes);

        $username = $authenticatedUser->{$attributes[$key]};
        $username = (is_array($username) ? Arr::get($username, 0) : $username);

        return [$key, $username];
    }

    /**
     * @param array $credentials
     * @return Model|null
     */
    public function retrieveByCredentials(array $credentials)
    {
        $authenticatedUser = $this->authenticateWithCredentials($credentials);
        // Check if we've authenticated and retrieved an AD user.
        if ($authenticatedUser instanceof AdLdapUser && $this->user = $authenticatedUser) {
            // Retrieve the password from the submitted credentials.
            $password = $this->getPasswordFromCredentials($credentials);
            // Construct / retrieve the eloquent model from our Adldap user.
            $model = $this->getModelFromAdldap($authenticatedUser, $password);
            // Perform other authenticated tasks.
            $this->handleAuthenticatedWithCredentials($authenticatedUser, $model);

            return $model;
        }
        if ($this->getLoginFallback()) {
            return $this->retrieveByFallback($credentials);
        }
    }

    /**
     * Return the user from the DB table if the exist
     * Will not return if the DB user is an LDAP user
     * @param array $credentials
     * @return null
     */
    protected function retrieveByFallback(array $credentials)
    {
        $model = $this->getEloquentUserByCredentials($credentials);

        //Check the user is not an LDAP user, this means they are no longer
        //a valid LDAP user and shouldnt not be given access
        if (!is_null($model) && !$model->ldap_user) {

            return $model;
        }

    }

    /**
     * Check if the specified user is in the specified LDAP group.
     * @param AdLdapUser $authenticatedUser The adldap user to check.
     * @return boolean
     * @throws LdapGroupException Thrown when the user is not authorized.
     */
    protected function checkLDAPGroups(AdLdapUser $authenticatedUser)
    {
        $group = $this->groupFinder->find($authenticatedUser);

        if ($group !== null) {
            $this->chosenGroup = $group;

            return true;
        }

        // If the user is not in the correct group, then we need to check the
        // users table to see if they have been given explicit access. At
        // this point the user has already been authorized by ldap
        $this->checkIfLdapUserHasBeenGivenExplicitAccess($authenticatedUser);

        return true;
    }

    /**
     * Retrieve a user by the given credentials.
     * @param AdLdapUser $authenticatedUser User to check.
     * @return void
     * @throws LdapGroupException Thrown when the user is not explicitly given access.
     */
    public function checkIfLdapUserHasBeenGivenExplicitAccess(AdLdapUser $authenticatedUser)
    {
        $user = $this->getEloquentUserFromLdapUser($authenticatedUser);
        $user = $this->removeUserIfNoAccess($user);

        if (is_null($user)) {
            throw new LdapGroupException('You do not have permission to access this CMS.');
        }
    }

    /**
     * Gets the eloquent user from the ldap user.
     * @param AdLdapUser $authenticatedUser Adldap to get Eloquent user.
     * @return DrpLdapUser
     */
    protected function getEloquentUserFromLdapUser(AdLdapUser $authenticatedUser)
    {
        $credentials = $this->getDatabaseCredentials($authenticatedUser);

        return $this->getEloquentUserByCredentials($credentials);
    }

    /**
     * Set allowed groups to check when logging in.
     * @param mixed $allowedGroups Groups to set on finder.
     * @return void
     */
    public function setAllowedGroups($allowedGroups)
    {
        $this->groupFinder->setAllowedGroups($allowedGroups);
    }

    /**
     * Get the Ldap credentials for a database lookup
     * @param AdLdapUser $authenticatedUser User to get credentials from.
     * @return array
     */
    protected function getDatabaseCredentials(AdLdapUser $authenticatedUser)
    {
        // Get the username input attributes
        $attributes = $this->getUsernameAttribute();
        // Get the input key
        $key = key($attributes);
        // Get the username from the AD model.
        $username = $authenticatedUser->{$attributes[$key]};

        // Make sure we retrieve the first username
        // result if it's an array.
        if (is_array($username)) {
            $username = array_get($username, 0);
        }

        $credentials = [
            $key => $username
        ];

        return $credentials;
    }

    /**
     * Removes the user record if the user doesn't have explicit access.
     * @param DrpLdapUser $user Model to delete if it doesn't have access.
     * @return Model|null
     */
    public function removeUserIfNoAccess(DrpLdapUser $user = null)
    {
        if (!$user instanceof Model) {
            return null;
        }

        if ($user->isExplicitLdapUser()) {
            return $user;
        }

        $user->delete();

        return null;
    }

    /**
     * @param array $credentials
     * @return mixed
     */
    protected function getEloquentUserByCredentials(array $credentials)
    {
        $query = $this->createModel()->newQuery();
        foreach ($credentials as $key => $value) {
            if (!str_contains($key, 'password')) {
                $query->where($key, $value);
            }
        }

        return $query->first();
    }

    /**
     * Checks if a user is deleted and will restore to have group access
     * @param User $user
     * @internal param $model
     */
    protected function restoreUserIfGroupAccess($user)
    {
        $user->setExplicitAccess(false);
        $user->restore();
    }
}
