<?php

namespace Drp\LdapAuthentication;

use Illuminate\Support\ServiceProvider;
use Adldap\Laravel\Exceptions\ConfigurationMissingException;
use Illuminate\Support\Facades\Auth;
use Adldap\Laravel\Commands\Import;
use Illuminate\Contracts\Hashing\Hasher;
use Adldap\Adldap;
use Adldap\Connections\Provider;
use Adldap\Contracts\AdldapInterface;
use Adldap\Contracts\Connections\ConnectionInterface;
use Adldap\Contracts\Schemas\SchemaInterface;

/**
 * Class DrpldapServiceProvider
 * @package Drp\LdapAuthentication
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class DrpldapServiceProvider extends ServiceProvider
{

    /**
     * Indicates if loading of the provider is deferred.
     * @var boolean
     */
    protected $defer = false;

    /**
     * Run service provider boot operations.
     * @return void
     */
    public function boot()
    {
        $this->bootLdap();
        $this->bootAuth();
    }

    /**
     * Register the service provider.
     * @return void
     */
    public function register()
    {
        // Bind the Adldap instance to the IoC
        $this->app->singleton('adldap', function () {
            $config = config('adldap');

            // Verify configuration exists.
            if (is_null($config)) {
                $message = 'Adldap configuration could not be found. `php artisan vendor:publish --tag="adldap"`.';
                throw new ConfigurationMissingException($message);
            }

            return $this->addProviders($this->newAdldap(), $config['connections']);
        });

        //Bind the group finder to the application using the config
        $this->app->bind(GroupFinder::class, function () {
            $config = config('adldap');

            return new  GroupFinder($config);
        });

        // Bind the Adldap contract to the Adldap object
        // in the IoC for dependency injection.
        $this->app->singleton(AdldapInterface::class, 'adldap');
    }

    /**
     * @return void
     */
    protected function bootAuth()
    {
        $auth = __DIR__ . '/../config/auth.php';

        // Add publishable configuration.
        $this->publishes([
            $auth => config_path('adldap_auth.php'),
        ], 'adldap');

        $auth = $this->getAuth();

        $this->commands([Import::class]);

        if (method_exists($auth, 'provider')) {

            // If the provider method exists, we're running Laravel 5.2.
            // Register the adldap auth user provider.
            $auth->provider('ldap', function ($app, array $config) {
                return $this->newAdldapAuthUserProvider($app['hash'], $config['model']);
            });

            return;
        }
        // Otherwise we're using 5.0 || 5.1
        // Extend Laravel authentication with Adldap driver.
        $auth->extend('ldap', function ($app) {
            return $this->newAdldapAuthUserProvider($app['hash'], $app['config']['auth.model']);
        });

    }

    /**
     * @return void
     */
    protected function bootLdap()
    {
        $this->publishes([
            __DIR__ . '/../database/migrations/' => database_path('migrations')
        ], 'migrations');

        $config = __DIR__ . '/../config/config.php';

        $this->publishes([
            $config => config_path('adldap.php'),
        ], 'adldap');

    }

    /**
     * Get the services provided by the provider.
     * @return array
     */
    public function provides()
    {
        return ['auth'];
    }

    /**
     * Adds providers to the specified Adldap instance.
     * @param Adldap $adldap
     * @param array $connections
     * @throws \Adldap\Exceptions\ConnectionException
     * @return Adldap
     */
    protected function addProviders(Adldap $adldap, array $connections = [])
    {
        // Go through each connection and construct a Provider.
        collect($connections)->each(function ($settings, $name) use ($adldap) {
            // Create a new provider.
            $provider = $this->newProvider(
                $settings['connection_settings'],
                new $settings['connection'](),
                new $settings['schema']()
            );

            // Try connecting to the provider if `auto_connect` is true.
            if ($settings['auto_connect'] === true) {
                $provider->connect();
            }

            // Add the provider to the Adldap container.
            $adldap->addProvider($name, $provider);
        });

        return $adldap;
    }

    /**
     * Returns a new Adldap instance.
     * @return Adldap
     */
    protected function newAdldap()
    {
        return new Adldap();
    }

    /**
     * Returns a new Provider instance.
     * @param array $configuration
     * @param ConnectionInterface|null $connection
     * @param SchemaInterface $schema
     * @return Provider
     */
    protected function newProvider(
        $configuration = [],
        ConnectionInterface $connection = null,
        SchemaInterface $schema = null
    ) {
        return new Provider($configuration, $connection, $schema);
    }

    /**
     * Returns a new instance of the AdldapAuthUserProvider.
     * @param Hasher $hasher
     * @param string $model
     * @return AdldapAuthUserProvider
     */
    protected function newAdldapAuthUserProvider(Hasher $hasher, $model)
    {
        return new DrpldapAuthUserProvider($hasher, $model, $this->app[GroupFinder::class]);
    }

    /**
     * Returns the root Auth instance.
     * @return mixed
     */
    protected function getAuth()
    {
        return Auth::getFacadeRoot();
    }
}
